@@ -24,9 +24,12 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
+
 import math
 
 from libqtile.layout.base import _SimpleLayoutBase
+from libqtile.log_utils import logger
 
 
 class Matrix(_SimpleLayoutBase):
@@ -37,16 +40,22 @@ class Matrix(_SimpleLayoutBase):
     """
 
     defaults = [
-        ("border_focus", "#0000ff", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused windows."),
+        ("border_focus", "#0000ff", "Border colour(s) for the focused window."),
+        ("border_normal", "#000000", "Border colour(s) for un-focused windows."),
         ("border_width", 1, "Border width."),
+        ("columns", 2, "Number of columns"),
         ("margin", 0, "Margin of the layout (int or list of ints [N E S W])"),
     ]
 
-    def __init__(self, columns=2, **config):
+    def __init__(self, _columns: int | None = None, **config):
         _SimpleLayoutBase.__init__(self, **config)
         self.add_defaults(Matrix.defaults)
-        self.columns = columns
+        if _columns:
+            logger.warning(
+                "The use of a positional argument in Matrix is deprecated. "
+                "Please update your config to use columns=..."
+            )
+            self.columns = _columns
 
     @property
     def rows(self):
@@ -56,7 +65,7 @@ class Matrix(_SimpleLayoutBase):
     @property
     def row(self):
         """Calc row index of current client"""
-        return (self.clients.current_index // self.columns)
+        return self.clients.current_index // self.columns
 
     @property
     def column(self):
@@ -65,10 +74,7 @@ class Matrix(_SimpleLayoutBase):
 
     def info(self):
         d = _SimpleLayoutBase.info(self)
-        d["rows"] = [
-            [win.name for win in self.get_row(i)]
-            for i in range(self.rows)
-        ]
+        d["rows"] = [[win.name for win in self.get_row(i)] for i in range(self.rows)]
         d["current_window"] = self.column, self.row
         return d
 
@@ -80,17 +86,12 @@ class Matrix(_SimpleLayoutBase):
     def get_row(self, row):
         """Get all clients in given row"""
         assert row < self.rows
-        return self.clients[
-            row * self.columns: row * self.columns + self.columns
-        ]
+        return self.clients[row * self.columns : row * self.columns + self.columns]
 
     def get_column(self, column):
         """Get all clients in given column"""
         assert column < self.columns
-        return [
-            self.clients[i]
-            for i in range(column, len(self.clients), self.columns)
-        ]
+        return [self.clients[i] for i in range(column, len(self.clients), self.columns)]
 
     def add(self, client):
         """Add client to Layout.
