@@ -34,6 +34,7 @@
 # SOFTWARE.
 
 import math
+from collections import namedtuple
 
 from libqtile.layout.base import _SimpleLayoutBase
 
@@ -46,7 +47,8 @@ class MonadTall(_SimpleLayoutBase):
     A main pane that contains a single window takes up a vertical portion of
     the screen_rect based on the ratio setting. This ratio can be adjusted with
     the ``cmd_grow_main`` and ``cmd_shrink_main`` or, while the main pane is in
-    focus, ``cmd_grow`` and ``cmd_shrink``.
+    focus, ``cmd_grow`` and ``cmd_shrink``. You may also set the ratio directly
+    with ``cmd_set_ratio``.
 
     ::
 
@@ -142,36 +144,47 @@ class MonadTall(_SimpleLayoutBase):
 
     _left = 0
     _right = 1
-    _med_ratio = 0.5
 
     defaults = [
-        ("border_focus", "#ff0000", "Border colour for the focused window."),
-        ("border_normal", "#000000", "Border colour for un-focused windows."),
+        ("border_focus", "#ff0000", "Border colour(s) for the focused window."),
+        ("border_normal", "#000000", "Border colour(s) for un-focused windows."),
         ("border_width", 2, "Border width."),
         ("single_border_width", None, "Border width for single window"),
         ("single_margin", None, "Margin size for single window"),
         ("margin", 0, "Margin of the layout"),
-        ("ratio", .5,
-            "The percent of the screen-space the master pane should occupy "
-            "by default."),
-        ("min_ratio", .25,
-            "The percent of the screen-space the master pane should occupy "
-            "at minimum."),
-        ("max_ratio", .75,
-            "The percent of the screen-space the master pane should occupy "
-            "at maximum."),
-        ("min_secondary_size", 85,
-            "minimum size in pixel for a secondary pane window "),
-        ("align", _left, "Which side master plane will be placed "
-            "(one of ``MonadTall._left`` or ``MonadTall._right``)"),
-        ("change_ratio", .05, "Resize ratio"),
+        (
+            "ratio",
+            0.5,
+            "The percent of the screen-space the master pane should occupy " "by default.",
+        ),
+        (
+            "min_ratio",
+            0.25,
+            "The percent of the screen-space the master pane should occupy " "at minimum.",
+        ),
+        (
+            "max_ratio",
+            0.75,
+            "The percent of the screen-space the master pane should occupy " "at maximum.",
+        ),
+        ("min_secondary_size", 85, "minimum size in pixel for a secondary pane window "),
+        (
+            "align",
+            _left,
+            "Which side master plane will be placed "
+            "(one of ``MonadTall._left`` or ``MonadTall._right``)",
+        ),
+        ("change_ratio", 0.05, "Resize ratio"),
         ("change_size", 20, "Resize change in pixels"),
-        ("new_client_position", "after_current",
-            "Place new windows : "
+        (
+            "new_client_position",
+            "after_current",
+            "Place new windows: "
             " after_current - after the active window."
             " before_current - before the active window,"
             " top - at the top of the stack,"
-            " bottom - at the bottom of the stack,"),
+            " bottom - at the bottom of the stack,",
+        ),
     ]
 
     def __init__(self, **config):
@@ -183,6 +196,7 @@ class MonadTall(_SimpleLayoutBase):
             self.single_margin = self.margin
         self.relative_sizes = []
         self.screen_rect = None
+        self.default_ratio = self.ratio
 
     @property
     def focused(self):
@@ -214,6 +228,12 @@ class MonadTall(_SimpleLayoutBase):
         self.do_normalize = True
         return self.clients.remove(client)
 
+    def cmd_set_ratio(self, ratio):
+        "Directly set the main pane ratio"
+        ratio = min(self.max_ratio, ratio)
+        self.ratio = max(self.min_ratio, ratio)
+        self.group.layout_all()
+
     def cmd_normalize(self, redraw=True):
         "Evenly distribute screen-space among secondary clients"
         n = len(self.clients) - 1  # exclude main client, 0
@@ -225,16 +245,16 @@ class MonadTall(_SimpleLayoutBase):
             self.group.layout_all()
         self.do_normalize = False
 
-    def cmd_reset(self, redraw=True):
+    def cmd_reset(self, ratio=None, redraw=True):
         "Reset Layout."
-        self.ratio = self._med_ratio
+        self.ratio = ratio or self.default_ratio
         if self.align == self._right:
             self.align = self._left
         self.cmd_normalize(redraw)
 
     def _maximize_main(self):
         "Toggle the main pane between min and max size"
-        if self.ratio <= self._med_ratio:
+        if self.ratio <= 0.5 * (self.max_ratio + self.min_ratio):
             self.ratio = self.max_ratio
         else:
             self.ratio = self.min_ratio
@@ -249,15 +269,14 @@ class MonadTall(_SimpleLayoutBase):
         # total height of maximized secondary
         maxed_size = self.group.screen.dheight - collapsed_size
         # if maximized or nearly maximized
-        if abs(
-            self._get_absolute_size_from_relative(self.relative_sizes[nidx]) -
-            maxed_size
-        ) < self.change_size:
+        if (
+            abs(self._get_absolute_size_from_relative(self.relative_sizes[nidx]) - maxed_size)
+            < self.change_size
+        ):
             # minimize
             self._shrink_secondary(
-                self._get_absolute_size_from_relative(
-                    self.relative_sizes[nidx]
-                ) - self.min_secondary_size
+                self._get_absolute_size_from_relative(self.relative_sizes[nidx])
+                - self.min_secondary_size
             )
         # otherwise maximize
         else:
@@ -338,14 +357,11 @@ class MonadTall(_SimpleLayoutBase):
             # secondary client
             width = width_shared - 2 * self.border_width
             # ypos is the sum of all clients above it
-            ypos = self.screen_rect.y + \
-                self._get_absolute_size_from_relative(
-                    sum(self.relative_sizes[:cidx - 1])
-                )
-            # get height from precalculated height list
-            height = self._get_absolute_size_from_relative(
-                self.relative_sizes[cidx - 1]
+            ypos = self.screen_rect.y + self._get_absolute_size_from_relative(
+                sum(self.relative_sizes[: cidx - 1])
             )
+            # get height from precalculated height list
+            height = self._get_absolute_size_from_relative(self.relative_sizes[cidx - 1])
             # fix double margin
             if cidx > 1:
                 ypos -= self.margin
@@ -371,26 +387,28 @@ class MonadTall(_SimpleLayoutBase):
                 px,
                 margin=[
                     self.margin,
-                    2*self.border_width,
-                    self.margin + 2*self.border_width,
-                    self.margin
+                    2 * self.border_width,
+                    self.margin + 2 * self.border_width,
+                    self.margin,
                 ],
             )
 
     def info(self):
         d = _SimpleLayoutBase.info(self)
-        d.update(dict(
-            main=d['clients'][0] if self.clients else None,
-            secondary=d['clients'][1::] if self.clients else []))
+        d.update(
+            dict(
+                main=d["clients"][0] if self.clients else None,
+                secondary=d["clients"][1::] if self.clients else [],
+            )
+        )
         return d
 
     def get_shrink_margin(self, cidx):
         "Return how many remaining pixels a client can shrink"
         return max(
             0,
-            self._get_absolute_size_from_relative(
-                self.relative_sizes[cidx]
-            ) - self.min_secondary_size
+            self._get_absolute_size_from_relative(self.relative_sizes[cidx])
+            - self.min_secondary_size,
         )
 
     def shrink(self, cidx, amt):
@@ -402,12 +420,10 @@ class MonadTall(_SimpleLayoutBase):
         # get max resizable amount
         margin = self.get_shrink_margin(cidx)
         if amt > margin:  # too much
-            self.relative_sizes[cidx] -= \
-                self._get_relative_size_from_absolute(margin)
+            self.relative_sizes[cidx] -= self._get_relative_size_from_absolute(margin)
             return amt - margin
         else:
-            self.relative_sizes[cidx] -= \
-                self._get_relative_size_from_absolute(amt)
+            self.relative_sizes[cidx] -= self._get_relative_size_from_absolute(amt)
             return 0
 
     def shrink_up(self, cidx, amt):
@@ -518,27 +534,14 @@ class MonadTall(_SimpleLayoutBase):
             # get size index
             idx = self.focused - 1
             # shrink up and down
-            left -= half_change_size - self.shrink_up_shared(
-                idx,
-                half_change_size
-            )
-            left -= half_change_size - self.shrink_down_shared(
-                idx,
-                half_change_size
-            )
-            left -= half_change_size - self.shrink_up_shared(
-                idx,
-                half_change_size
-            )
-            left -= half_change_size - self.shrink_down_shared(
-                idx,
-                half_change_size
-            )
+            left -= half_change_size - self.shrink_up_shared(idx, half_change_size)
+            left -= half_change_size - self.shrink_down_shared(idx, half_change_size)
+            left -= half_change_size - self.shrink_up_shared(idx, half_change_size)
+            left -= half_change_size - self.shrink_down_shared(idx, half_change_size)
         # calculate how much shrinkage took place
         diff = amt - left
         # grow client by diff amount
-        self.relative_sizes[self.focused - 1] += \
-            self._get_relative_size_from_absolute(diff)
+        self.relative_sizes[self.focused - 1] += self._get_relative_size_from_absolute(diff)
 
     def cmd_grow(self):
         """Grow current window
@@ -642,8 +645,7 @@ class MonadTall(_SimpleLayoutBase):
             self.grow_up_shared(idx, half_change)
             self.grow_down_shared(idx, half_change)
         # shrink client by total change
-        self.relative_sizes[self.focused - 1] -= \
-            self._get_relative_size_from_absolute(change)
+        self.relative_sizes[self.focused - 1] -= self._get_relative_size_from_absolute(change)
 
     cmd_next = _SimpleLayoutBase.next
     cmd_previous = _SimpleLayoutBase.previous
@@ -687,7 +689,8 @@ class MonadTall(_SimpleLayoutBase):
         """Get closest window to a point x,y"""
         target = min(
             clients,
-            key=lambda c: math.hypot(c.info()["x"] - x, c.info()["y"] - y)
+            key=lambda c: math.hypot(c.x - x, c.y - y),
+            default=self.clients.current_client,
         )
         return target
 
@@ -701,7 +704,7 @@ class MonadTall(_SimpleLayoutBase):
         """Swap current window with closest window to the left"""
         win = self.clients.current_client
         x, y = win.x, win.y
-        candidates = [c for c in self.clients if c.info()['x'] < x]
+        candidates = [c for c in self.clients if c.info()["x"] < x]
         target = self._get_closest(x, y, candidates)
         self.cmd_swap(win, target)
 
@@ -709,7 +712,7 @@ class MonadTall(_SimpleLayoutBase):
         """Swap current window with closest window to the right"""
         win = self.clients.current_client
         x, y = win.x, win.y
-        candidates = [c for c in self.clients if c.info()['x'] > x]
+        candidates = [c for c in self.clients if c.info()["x"] > x]
         target = self._get_closest(x, y, candidates)
         self.cmd_swap(win, target)
 
@@ -864,15 +867,14 @@ class MonadWide(MonadTall):
         # total width of maximized secondary
         maxed_size = self.screen_rect.width - collapsed_size
         # if maximized or nearly maximized
-        if abs(
-            self._get_absolute_size_from_relative(self.relative_sizes[nidx]) -
-            maxed_size
-        ) < self.change_size:
+        if (
+            abs(self._get_absolute_size_from_relative(self.relative_sizes[nidx]) - maxed_size)
+            < self.change_size
+        ):
             # minimize
             self._shrink_secondary(
-                self._get_absolute_size_from_relative(
-                    self.relative_sizes[nidx]
-                ) - self.min_secondary_size
+                self._get_absolute_size_from_relative(self.relative_sizes[nidx])
+                - self.min_secondary_size
             )
         # otherwise maximize
         else:
@@ -907,14 +909,11 @@ class MonadWide(MonadTall):
             # secondary client
             height = height_shared - 2 * self.border_width
             # xpos is the sum of all clients left of it
-            xpos = self.screen_rect.x + \
-                self._get_absolute_size_from_relative(
-                    sum(self.relative_sizes[:cidx - 1])
-                )
-            # get width from precalculated width list
-            width = self._get_absolute_size_from_relative(
-                self.relative_sizes[cidx - 1]
+            xpos = self.screen_rect.x + self._get_absolute_size_from_relative(
+                sum(self.relative_sizes[: cidx - 1])
             )
+            # get width from precalculated width list
+            width = self._get_absolute_size_from_relative(self.relative_sizes[cidx - 1])
             # fix double margin
             if cidx > 1:
                 xpos -= self.margin
@@ -940,8 +939,8 @@ class MonadWide(MonadTall):
                 px,
                 margin=[
                     self.margin,
-                    self.margin + 2*self.border_width,
-                    2*self.border_width,
+                    self.margin + 2 * self.border_width,
+                    2 * self.border_width,
                     self.margin,
                 ],
             )
@@ -979,8 +978,7 @@ class MonadWide(MonadTall):
             self.grow_up_shared(idx, half_change)
             self.grow_down_shared(idx, half_change)
         # shrink client by total change
-        self.relative_sizes[self.focused - 1] -= \
-            self._get_relative_size_from_absolute(change)
+        self.relative_sizes[self.focused - 1] -= self._get_relative_size_from_absolute(change)
 
     def cmd_swap_left(self):
         """Swap current window with closest window to the down"""
@@ -1004,3 +1002,300 @@ class MonadWide(MonadTall):
             self.cmd_swap_right()
         elif self.align == self._down:
             self.cmd_swap_left()
+
+
+class MonadThreeCol(MonadTall):
+    """Emulate the behavior of XMonad's ThreeColumns layout.
+
+    A layout similar to tall but with three columns. With an ultra wide display
+    this layout can be used for a huge main window - ideally at the center of the
+    screen - and up to six reasonable sized secondary windows.
+
+    Main-Pane:
+
+    A main pane that contains a single window takes up a vertical portion of
+    the screen_rect based on the ratio setting. This ratio can be adjusted with
+    the ``cmd_grow_main`` and ``cmd_shrink_main`` or, while the main pane is in
+    focus, ``cmd_grow`` and ``cmd_shrink``. The main pane can also be centered.
+
+    ::
+
+        ---------------------------    ---------------------------
+        |           |      |      |    |      |           |      |
+        |           |      |      |    |      |           |      |
+        |           |      |      |    |      |           |      |
+        |           |      |      |    |      |           |      |
+        |           |      |      |    |      |           |      |
+        |           |      |      |    |      |           |      |
+        ---------------------------    ---------------------------
+
+    Secondary-panes:
+
+    Occupying the rest of the screen_rect are one or more secondary panes.  The
+    secondary panes will be divided into two columns and share the vertical space
+    of each column. However they can be resized at will with the ``cmd_grow`` and
+    ``cmd_shrink`` methods. The other secondary panes will adjust their sizes to
+    smoothly fill all of the space.
+
+    ::
+
+        ---------------------------    ---------------------------
+        |           |      |      |    |           |______|      |
+        |           |______|      |    |           |      |      |
+        |           |      |______|    |           |      |______|
+        |           |______|      |    |           |      |      |
+        |           |      |      |    |           |______|      |
+        |           |      |      |    |           |      |      |
+        ---------------------------    ---------------------------
+
+    Panes can be moved with the ``cmd_shuffle_up`` and ``cmd_shuffle_down``
+    methods. As mentioned the main pane is considered the top of the stack;
+    moving up is counter-clockwise and moving down is clockwise. A secondary
+    pane can also be promoted to the main pane with the ``cmd_swap_main``
+    method.
+
+    Normalizing/Resetting:
+
+    To restore all secondary client windows to their default size ratios
+    use the ``cmd_normalize`` method.
+
+    To reset all client windows to their default sizes, including the primary
+    window, use the ``cmd_reset`` method.
+
+    Maximizing:
+
+    To maximized a client window simply use the ``cmd_maximize`` on a focused
+    client.
+    """
+
+    defaults = [
+        ("main_centered", True, "Place the main pane at the center of the screen"),
+        (
+            "new_client_position",
+            "top",
+            "Place new windows: "
+            " after_current - after the active window."
+            " before_current - before the active window,"
+            " top - at the top of the stack,"
+            " bottom - at the bottom of the stack,",
+        ),
+    ]
+
+    __column = namedtuple("__column", "name count start end")
+
+    def __init__(self, **config):
+        MonadTall.__init__(self, **config)
+        self.add_defaults(MonadThreeCol.defaults)
+
+    def _configure_specific(self, client, screen_rect, border_color, index):
+        """Specific configuration for xmonad three columns."""
+        if index == 0:
+            self._configure_main(client)
+        elif self._get_column(index - 1).name == "left":
+            self._configure_left(client, index)
+        else:
+            self._configure_right(client, index)
+
+    def _configure_main(self, client):
+        """Configure the main client"""
+        width = self._get_main_width()
+        height = self.screen_rect.height
+        left = self.screen_rect.x
+        top = self.screen_rect.y
+
+        if self.main_centered and len(self.clients) > 2:
+            left += (self.screen_rect.width - width) // 2
+
+        self._place_client(client, left, top, width, height)
+
+    def _configure_left(self, client, index):
+        """Configure the left column"""
+        width = self._get_secondary_widths()[0]
+        height = self._get_secondary_height(index)
+        left = self.screen_rect.x
+        top = self.screen_rect.y + self._get_relative_sizes_above(index)
+
+        if not self.main_centered or len(self.clients) == 2:
+            left += self._get_main_width()
+
+        self._place_client(client, left, top, width, height)
+
+    def _configure_right(self, client, index):
+        """Configure the right column"""
+        widths = self._get_secondary_widths()
+        height = self._get_secondary_height(index)
+        left = self.screen_rect.x + widths[0] + self._get_main_width()
+        top = self.screen_rect.y + self._get_relative_sizes_above(index)
+
+        self._place_client(client, left, top, widths[1], height)
+
+    def _get_main_width(self):
+        """Calculate the main client's width"""
+        return int(self.screen_rect.width * self.ratio)
+
+    def _get_secondary_widths(self):
+        """Calculate secondary clients' widths"""
+        width = self.screen_rect.width - self._get_main_width()
+        if len(self.clients) == 2:
+            return [width, 0]
+
+        return self._split_integer(width, 2)
+
+    def _get_secondary_height(self, index):
+        """Return the height of the provided index"""
+        return self.relative_sizes[index - 1]
+
+    def _get_relative_sizes_above(self, index):
+        """Return the sum of the heights of all clients above the provided index"""
+        column = self._get_column(index - 1)
+        return sum(self.relative_sizes[column.start : index - 1])
+
+    def _place_client(self, client, left, top, width, height):
+        """Place a client on the screen
+
+        Will prevent double margins by applying east and south margins only
+        when the client is the rightmost or the bottommost window.
+        """
+        rightmost = left + width - self.screen_rect.x >= self.screen_rect.width
+        bottommost = top + height - self.screen_rect.y >= self.screen_rect.height
+        margin = [self.margin] * 4
+        if not rightmost:
+            margin[1] = 0
+        if not bottommost:
+            margin[2] = 0
+
+        client.place(
+            left,
+            top,
+            width - 2 * self.border_width,
+            height - 2 * self.border_width,
+            self.border_width,
+            self.border_focus if client.has_focus else self.border_normal,
+            margin=margin,
+        )
+
+    def cmd_normalize(self, redraw=True):
+        """Evenly distribute screen-space among secondary clients"""
+        if self.screen_rect is not None:
+            self.relative_sizes = []
+
+            height = self.screen_rect.height
+            left, right = self._get_columns()
+
+            if left.count > 0:
+                self.relative_sizes += self._split_integer(height, left.count)
+            if right.count > 0:
+                self.relative_sizes += self._split_integer(height, right.count)
+
+        if redraw:
+            self.group.layout_all()
+        self.do_normalize = False
+
+    def cmd_swap_main(self):
+        """Swap current window to main pane"""
+        self.cmd_swap(self.clients.current_client, self.clients[0])
+
+    def _maximize_secondary(self):
+        """Maximize the focused secondary pane"""
+        focused = self.focused - 1
+        column = self._get_column(focused)
+        if column.count == 1:
+            return
+
+        max_height = self.screen_rect.height - ((column.count - 1) * self.min_secondary_size)
+        for i in range(column.start, column.end):
+            self.relative_sizes[i] = max_height if i == focused else self.min_secondary_size
+
+    def _grow_secondary(self, amt):
+        """Grow the focused client in the secondary pane"""
+        self._resize_secondary(amt)
+
+    def _shrink_secondary(self, amt):
+        """Shrink the focused client in the secondary pane"""
+        self._resize_secondary(-amt)
+
+    def _resize_secondary(self, amt):
+        """Resize the focused secondary client
+
+        If amt is positive, the client will grow. Conversely, if it's negative,
+        the client will shrink. All other clients in the same column will get
+        grown/shrunk so to accommodate the new height.
+        """
+        focused = self.focused - 1
+        column = self._get_column(focused)
+
+        if column.count == 1:
+            return
+
+        # Resizing is accomplished by doing the following:
+        # - calculate how much each client in the column must shrink/grow
+        #   so that the focused window can grow/shrink.
+        # - iterate over all clients in the column and change their height
+        #   (grow or shrink) as long as they can still be resized (both main
+        #   and secondary windows).
+        min_height = self.min_secondary_size
+        idx = column.start
+        step = amt // (column.count - 1)
+        visited = 0
+        while amt != 0:
+            if idx != focused:
+                focused_new_height = self.relative_sizes[focused] + step
+                new_height = self.relative_sizes[idx] - step
+                if focused_new_height >= min_height and new_height >= min_height:
+                    self.relative_sizes[focused] += step
+                    self.relative_sizes[idx] -= step
+                    amt -= step
+                    visited += 1
+
+            idx += 1
+            if idx == column.end:
+                if visited == 0:
+                    break
+
+                idx = column.start
+                visited = 0
+
+        self.group.layout_all()
+
+    def _get_column(self, index):
+        """Get the column containing the provided index"""
+        left, right = self._get_columns()
+        return left if index < left.count else right
+
+    def _get_columns(self):
+        """Get all columns"""
+        clients = len(self.clients) - 1
+        clients = (clients // 2 + clients % 2, clients // 2)
+        return (
+            MonadThreeCol.__column(
+                name="left",
+                count=clients[0],
+                start=0,
+                end=clients[0],
+            ),
+            MonadThreeCol.__column(
+                name="right",
+                count=clients[1],
+                start=clients[0],
+                end=clients[0] + clients[1],
+            ),
+        )
+
+    def info(self):
+        left, right = self._get_columns()
+        d = MonadTall.info(self)
+        d.update(
+            secondary=dict(
+                left=d["clients"][1 : left.end + 1] if left.count > 0 else [],
+                right=d["clients"][right.start + 1 :] if right.count > 0 else [],
+            )
+        )
+        return d
+
+    @staticmethod
+    def _split_integer(value, parts):
+        """Divide an integer into equal parts and distribute the remainder"""
+        result = [value // parts] * parts
+        for i in range(value % parts):
+            result[i] += 1
+        return result
