@@ -19,6 +19,8 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
 import asyncio
 import glob
 import importlib
@@ -28,12 +30,19 @@ from collections import defaultdict
 from collections.abc import Sequence
 from random import randint
 from shutil import which
-from typing import List, Tuple, Union
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from typing import Union
+
+    ColorType = Union[str, tuple[int, int, int], tuple[int, int, int, float]]
+    ColorsType = Union[ColorType, list[ColorType]]
 
 try:
-    from dbus_next import Message, Variant  # type: ignore
-    from dbus_next.aio import MessageBus  # type: ignore
-    from dbus_next.constants import BusType, MessageType  # type: ignore
+    from dbus_next import Message, Variant
+    from dbus_next.aio import MessageBus
+    from dbus_next.constants import BusType, MessageType
+
     has_dbus = True
 except ImportError:
     has_dbus = False
@@ -66,57 +75,57 @@ def shuffle_down(lst):
         lst.append(c)
 
 
-ColorType = Union[str, Tuple[int, int, int], Tuple[int, int, int, float]]
-
-
-def rgb(x):
+def rgb(x: ColorType) -> tuple[float, float, float, float]:
     """
-        Returns a valid RGBA tuple.
+    Returns a valid RGBA tuple.
 
-        Here are some valid specifcations:
-            #ff0000
-            with alpha: #ff000080
-            ff0000
-            with alpha: ff0000.5
-            (255, 0, 0)
-            with alpha: (255, 0, 0, 0.5)
+    Here are some valid specifications:
+        #ff0000
+        with alpha: #ff000080
+        ff0000
+        with alpha: ff0000.5
+        (255, 0, 0)
+        with alpha: (255, 0, 0, 0.5)
+
+    Which is returned as (1.0, 0.0, 0.0, 0.5).
     """
     if isinstance(x, (tuple, list)):
         if len(x) == 4:
-            alpha = x[3]
+            alpha = x[-1]
         else:
-            alpha = 1
+            alpha = 1.0
         return (x[0] / 255.0, x[1] / 255.0, x[2] / 255.0, alpha)
     elif isinstance(x, str):
         if x.startswith("#"):
             x = x[1:]
         if "." in x:
-            x, alpha = x.split(".")
-            alpha = float("0." + alpha)
+            x, alpha_str = x.split(".")
+            alpha = float("0." + alpha_str)
         else:
-            alpha = 1
+            alpha = 1.0
         if len(x) not in (6, 8):
             raise ValueError("RGB specifier must be 6 or 8 characters long.")
-        vals = [int(i, 16) for i in (x[0:2], x[2:4], x[4:6])]
+        vals = tuple(int(i, 16) for i in (x[0:2], x[2:4], x[4:6]))
         if len(x) == 8:
             alpha = int(x[6:8], 16) / 255.0
-        vals.append(alpha)
-        return rgb(vals)
+        vals += (alpha,)  # type: ignore
+        return rgb(vals)  # type: ignore
     raise ValueError("Invalid RGB specifier.")
 
 
 def hex(x):
     r, g, b, _ = rgb(x)
-    return '#%02x%02x%02x' % (int(r * 255), int(g * 255), int(b * 255))
+    return "#%02x%02x%02x" % (int(r * 255), int(g * 255), int(b * 255))
 
 
-def has_transparency(colour: Union[ColorType, List[ColorType]]):
+def has_transparency(colour: ColorsType):
     """
     Returns True if the colour is not fully opaque.
 
     Where a list of colours is passed, returns True if any
     colour is not fully opaque.
     """
+
     def has_alpha(col):
         return rgb(col)[3] < 1
 
@@ -124,16 +133,16 @@ def has_transparency(colour: Union[ColorType, List[ColorType]]):
         return has_alpha(colour)
 
     elif isinstance(colour, list):
-        print([c for c in colour])
         return any([has_transparency(c) for c in colour])
 
     return False
 
 
-def remove_transparency(colour: Union[ColorType, List[ColorType]]):
+def remove_transparency(colour: ColorsType):
     """
     Returns a tuple of (r, g, b) with no alpha.
     """
+
     def remove_alpha(col):
         stripped = tuple(x * 255.0 for x in rgb(col)[:3])
         return stripped
@@ -161,11 +170,11 @@ def get_cache_dir():
     Returns the cache directory and create if it doesn't exists
     """
 
-    cache_directory = os.path.expandvars('$XDG_CACHE_HOME')
-    if cache_directory == '$XDG_CACHE_HOME':
+    cache_directory = os.path.expandvars("$XDG_CACHE_HOME")
+    if cache_directory == "$XDG_CACHE_HOME":
         # if variable wasn't set
         cache_directory = os.path.expanduser("~/.cache")
-    cache_directory = os.path.join(cache_directory, 'qtile')
+    cache_directory = os.path.join(cache_directory, "qtile")
     if not os.path.exists(cache_directory):
         os.makedirs(cache_directory)
     return cache_directory
@@ -182,9 +191,9 @@ def describe_attributes(obj, attrs, func=lambda x: x):
     for attr in attrs:
         value = getattr(obj, attr, None)
         if func(value):
-            pairs.append('%s=%s' % (attr, value))
+            pairs.append("%s=%s" % (attr, value))
 
-    return ', '.join(pairs)
+    return ", ".join(pairs)
 
 
 def import_class(module_path, class_name, fallback=None):
@@ -197,8 +206,7 @@ def import_class(module_path, class_name, fallback=None):
         module = importlib.import_module(module_path, __package__)
         return getattr(module, class_name)
     except ImportError as error:
-        logger.warning("Unmet dependencies for '%s.%s': %s", module_path,
-                       class_name, error)
+        logger.warning("Unmet dependencies for '%s.%s': %s", module_path, class_name, error)
         if fallback:
             logger.debug("%s", traceback.format_exc())
             return fallback(module_path, class_name)
@@ -235,16 +243,14 @@ def send_notification(title, message, urgent=False, timeout=10000, id=None):
     https://developer.gnome.org/notification-spec/
     """
     if not has_dbus:
-        logger.warning(
-            "dbus-next is not installed. Unable to send notifications."
-        )
+        logger.warning("dbus-next is not installed. Unable to send notifications.")
         return -1
 
     id = randint(10, 1000) if id is None else id
     urgency = 2 if urgent else 1
 
     try:
-        loop = asyncio.get_running_loop()
+        loop = asyncio.get_event_loop()
     except RuntimeError:
         logger.warning("Eventloop has not started. Cannot send notification.")
     else:
@@ -254,27 +260,30 @@ def send_notification(title, message, urgent=False, timeout=10000, id=None):
 
 
 async def _notify(title, message, urgency, timeout, id):
-    notification = ["qtile",  # Application name
-                    id,  # id
-                    "",  # icon
-                    title,  # summary
-                    message,  # body
-                    [],  # actions
-                    {"urgency": Variant("y", urgency)},  # hints
-                    timeout]  # timeout
+    notification = [
+        "qtile",  # Application name
+        id,  # id
+        "",  # icon
+        title,  # summary
+        message,  # body
+        [],  # actions
+        {"urgency": Variant("y", urgency)},  # hints
+        timeout,
+    ]  # timeout
 
-    bus, msg = await _send_dbus_message(True,
-                                        MessageType.METHOD_CALL,
-                                        "org.freedesktop.Notifications",
-                                        "org.freedesktop.Notifications",
-                                        "/org/freedesktop/Notifications",
-                                        "Notify",
-                                        "susssasa{sv}i",
-                                        notification)
+    bus, msg = await _send_dbus_message(
+        True,
+        MessageType.METHOD_CALL,
+        "org.freedesktop.Notifications",
+        "org.freedesktop.Notifications",
+        "/org/freedesktop/Notifications",
+        "Notify",
+        "susssasa{sv}i",
+        notification,
+    )
 
     if msg.message_type == MessageType.ERROR:
-        logger.warning("Unable to send notification. "
-                       "Is a notification server running?")
+        logger.warning("Unable to send notification. " "Is a notification server running?")
 
     # a new bus connection is made each time a notification is sent so
     # we disconnect when the notification is done
@@ -289,37 +298,37 @@ def guess_terminal(preference=None):
     elif isinstance(preference, Sequence):
         test_terminals += list(preference)
     test_terminals += [
-        'roxterm',
-        'sakura',
-        'hyper',
-        'alacritty',
-        'terminator',
-        'termite',
-        'gnome-terminal',
-        'konsole',
-        'xfce4-terminal',
-        'lxterminal',
-        'mate-terminal',
-        'kitty',
-        'yakuake',
-        'tilda',
-        'guake',
-        'eterm',
-        'st',
-        'urxvt',
-        'xterm',
-        'x-terminal-emulator',
+        "roxterm",
+        "sakura",
+        "hyper",
+        "alacritty",
+        "terminator",
+        "termite",
+        "gnome-terminal",
+        "konsole",
+        "xfce4-terminal",
+        "lxterminal",
+        "mate-terminal",
+        "kitty",
+        "yakuake",
+        "tilda",
+        "guake",
+        "eterm",
+        "st",
+        "urxvt",
+        "xterm",
+        "x-terminal-emulator",
     ]
 
     for terminal in test_terminals:
-        logger.debug('Guessing terminal: {}'.format(terminal))
+        logger.debug("Guessing terminal: {}".format(terminal))
         if not which(terminal, os.X_OK):
             continue
 
-        logger.info('Terminal found: {}'.format(terminal))
+        logger.info("Terminal found: {}".format(terminal))
         return terminal
 
-    logger.error('Default terminal has not been found.')
+    logger.error("Default terminal has not been found.")
 
 
 def scan_files(dirpath, *names):
@@ -333,17 +342,19 @@ def scan_files(dirpath, *names):
     ['/wallpapers/w2.jpg', '/wallpapers/w3.jpg']})
 
     """
+    dirpath = os.path.expanduser(dirpath)
     files = defaultdict(list)
 
     for name in names:
-        found = glob.glob(os.path.join(dirpath, '**', name), recursive=True)
+        found = glob.glob(os.path.join(dirpath, "**", name), recursive=True)
         files[name].extend(found)
 
     return files
 
 
-async def _send_dbus_message(session_bus, message_type, destination, interface,
-                             path, member, signature, body):
+async def _send_dbus_message(
+    session_bus, message_type, destination, interface, path, member, signature, body
+):
     """
     Private method to send messages to dbus via dbus_next.
 
@@ -360,19 +371,23 @@ async def _send_dbus_message(session_bus, message_type, destination, interface,
     bus = await MessageBus(bus_type=bus_type).connect()
 
     msg = await bus.call(
-        Message(message_type=message_type,
-                destination=destination,
-                interface=interface,
-                path=path,
-                member=member,
-                signature=signature,
-                body=body))
+        Message(
+            message_type=message_type,
+            destination=destination,
+            interface=interface,
+            path=path,
+            member=member,
+            signature=signature,
+            body=body,
+        )
+    )
 
     return bus, msg
 
 
-async def add_signal_receiver(callback, session_bus=False, signal_name=None,
-                              dbus_interface=None, bus_name=None, path=None):
+async def add_signal_receiver(
+    callback, session_bus=False, signal_name=None, dbus_interface=None, bus_name=None, path=None
+):
     """
     Helper function which aims to recreate python-dbus's add_signal_receiver
     method in dbus_next with asyncio calls.
@@ -380,10 +395,7 @@ async def add_signal_receiver(callback, session_bus=False, signal_name=None,
     Returns True if subscription is successful.
     """
     if not has_dbus:
-        logger.warning(
-            "dbus-next is not installed. "
-            "Unable to subscribe to signals"
-        )
+        logger.warning("dbus-next is not installed. " "Unable to subscribe to signals")
         return False
 
     match_args = {
@@ -391,20 +403,21 @@ async def add_signal_receiver(callback, session_bus=False, signal_name=None,
         "sender": bus_name,
         "member": signal_name,
         "path": path,
-        "interface": dbus_interface
+        "interface": dbus_interface,
     }
 
-    rule = ",".join("{}='{}'".format(k, v)
-                    for k, v in match_args.items() if v)
+    rule = ",".join("{}='{}'".format(k, v) for k, v in match_args.items() if v)
 
-    bus, msg = await _send_dbus_message(session_bus,
-                                        MessageType.METHOD_CALL,
-                                        "org.freedesktop.DBus",
-                                        "org.freedesktop.DBus",
-                                        "/org/freedesktop/DBus",
-                                        "AddMatch",
-                                        "s",
-                                        rule)
+    bus, msg = await _send_dbus_message(
+        session_bus,
+        MessageType.METHOD_CALL,
+        "org.freedesktop.DBus",
+        "org.freedesktop.DBus",
+        "/org/freedesktop/DBus",
+        "AddMatch",
+        "s",
+        rule,
+    )
 
     # Check if message sent successfully
     if msg.message_type == MessageType.METHOD_RETURN:
