@@ -29,27 +29,27 @@ class QuickExit(base._TextBox):
     """
 
     defaults = [
-        ('default_text', '[ shutdown ]', 'A text displayed as a button'),
-        ('countdown_format', '[ {} seconds ]', 'This text is showed when counting down.'),
-        ('timer_interval', 1, 'A countdown interval.'),
-        ('countdown_start', 5, 'Time to accept the second pushing.'),
+        ("default_text", "[ shutdown ]", "A text displayed as a button"),
+        ("countdown_format", "[ {} seconds ]", "This text is showed when counting down."),
+        ("timer_interval", 1, "A countdown interval."),
+        ("countdown_start", 5, "Time to accept the second pushing."),
     ]
 
     def __init__(self, widget=bar.CALCULATED, **config):
-        base._TextBox.__init__(self, '', widget, **config)
+        base._TextBox.__init__(self, "", widget, **config)
         self.add_defaults(QuickExit.defaults)
 
         self.is_counting = False
         self.text = self.default_text
         self.countdown = self.countdown_start
-        self.__call_later_funcs = []
+
+        self.add_callbacks({"Button1": self.cmd_trigger})
 
     def __reset(self):
         self.is_counting = False
         self.countdown = self.countdown_start
         self.text = self.default_text
-        for f in self.__call_later_funcs:
-            f.cancel()
+        self.timer.cancel()
 
     def update(self):
         if not self.is_counting:
@@ -57,20 +57,17 @@ class QuickExit(base._TextBox):
 
         self.countdown -= 1
         self.text = self.countdown_format.format(self.countdown)
-        func = self.timeout_add(self.timer_interval, self.update)
-        self.__call_later_funcs.append(func)
+        self.timer = self.timeout_add(self.timer_interval, self.update)
         self.draw()
 
         if self.countdown == 0:
             self.qtile.stop()
             return
 
-    def button_press(self, x, y, button):
-
+    def cmd_trigger(self):
         if not self.is_counting:
             self.is_counting = True
             self.update()
-            return
         else:
             self.__reset()
             self.draw()
